/*
 * Copyright (c) 2017, tobias
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package opensesim.gui;


import java.awt.MouseInfo;
import java.awt.Point;
import java.util.Iterator;
import java.util.Map;


import javax.swing.table.DefaultTableModel;

import opensesim.old_sesim.Account;
import opensesim.old_sesim.Order;
import opensesim.old_sesim.Order.OrderType;
import opensesim.trader.ManTrader.CreateOrderDialog;

/**
 *
 * @author tobias
 */
public class OpenOrdersList extends javax.swing.JPanel {

    private Account account;
    DefaultTableModel model;

    public final void updateModel() {
        if (null == account) {
            return;
        }



        int row = 0;

        Iterator<Map.Entry<Long, Order>> it = account.getOrders().entrySet().iterator();
        model.setRowCount(account.getOrders().size());
        while (it.hasNext()) {
            Map.Entry e = it.next();
            Long k = (Long) e.getKey();
            Order o = (Order) e.getValue();

            model.setValueAt(k, row, 0);

            model.setValueAt(((Order) o).getType().toString(), row, 1);

            model.setValueAt(((Order) o).getLimit(), row, 2);
            model.setValueAt(((Order) o).getExecuted(), row, 3);
            model.setValueAt(((Order) o).getAvaragePrice(), row, 4);
            model.setValueAt(((Order) o).getInitialVolume(), row, 5);
            model.setValueAt(((Order) o).getOrderStatus().toString(), row, 6);
            row++;
        }
        this.table.getRowSorter().allRowsChanged();

    }

    /**
     * Creates new form OrdersList
     */
    public OpenOrdersList() {
        initComponents();
        model = (DefaultTableModel) table.getModel();
        model.setRowCount(0);
        table.setFillsViewportHeight(true);
        updateModel();
        
    }

    public void initOrderList(Account account) {
        this.account = account;
        updateModel();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        ctxMenu = new javax.swing.JPopupMenu();
        ctxMenuCreateBuyOrder = new javax.swing.JMenuItem();
        ctxMenuCreateSellOrder = new javax.swing.JMenuItem();
        ctxMenuCancelOrder = new javax.swing.JMenuItem();
        ctxMenuModifyOder = new javax.swing.JMenuItem();
        jScrollPane1 = new javax.swing.JScrollPane();
        table = new javax.swing.JTable();

        ctxMenuCreateBuyOrder.setText("Create Buy Order");
        ctxMenuCreateBuyOrder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ctxMenuCreateBuyOrderActionPerformed(evt);
            }
        });
        ctxMenu.add(ctxMenuCreateBuyOrder);

        ctxMenuCreateSellOrder.setText("Create Sell Order");
        ctxMenuCreateSellOrder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ctxMenuCreateSellOrderActionPerformed(evt);
            }
        });
        ctxMenu.add(ctxMenuCreateSellOrder);

        ctxMenuCancelOrder.setText("Cancel Order");
        ctxMenuCancelOrder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ctxMenuCancelOrderActionPerformed(evt);
            }
        });
        ctxMenu.add(ctxMenuCancelOrder);

        ctxMenuModifyOder.setText("Modify Oder");
        ctxMenu.add(ctxMenuModifyOder);

        table.setAutoCreateRowSorter(true);
        table.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null}
            },
            new String [] {
                "ID", "Type", "Limit", "Vol. Executed", "Av. Price", "Volume", "Status"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Long.class, java.lang.String.class, java.lang.Double.class, java.lang.Double.class, java.lang.Double.class, java.lang.Double.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        table.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        table.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                tableMousePressed(evt);
            }
        });
        jScrollPane1.setViewportView(table);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1)
        );
    }// </editor-fold>//GEN-END:initComponents

    private void tableMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tableMousePressed

        Point point = evt.getPoint();
        int currentRow = table.rowAtPoint(point);
        if (currentRow != -1) {
            
            table.setRowSelectionInterval(currentRow, currentRow);
        } else {
            
        }

        int nbuttons = MouseInfo.getNumberOfButtons();
        int button = evt.getButton();
        if (!evt.isPopupTrigger()) {
            return;
        }

        ctxMenu.show(table, point.x,point.y);

    }//GEN-LAST:event_tableMousePressed

    private void createOrder(OrderType t) {
        CreateOrderDialog cd = new CreateOrderDialog(Globals.frame, true, account, t);
        //cd.initDialog(account);
        cd.setVisible(true);

    }


    private void ctxMenuCreateSellOrderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ctxMenuCreateSellOrderActionPerformed
        createOrder(OrderType.SELLLIMIT);

    }//GEN-LAST:event_ctxMenuCreateSellOrderActionPerformed

    private void ctxMenuCreateBuyOrderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ctxMenuCreateBuyOrderActionPerformed
        createOrder(OrderType.BUYLIMIT);
    }//GEN-LAST:event_ctxMenuCreateBuyOrderActionPerformed

    private void ctxMenuCancelOrderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ctxMenuCancelOrderActionPerformed
        int r = table.getSelectedRow();
        Long id = (Long) model.getValueAt(r, 0);
        
        Order co = account.getOrders().get(id);

        System.out.printf("Should cancel %d\n", id);
        Globals.se.cancelOrder(account.getID(), co);

    }//GEN-LAST:event_ctxMenuCancelOrderActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPopupMenu ctxMenu;
    private javax.swing.JMenuItem ctxMenuCancelOrder;
    private javax.swing.JMenuItem ctxMenuCreateBuyOrder;
    private javax.swing.JMenuItem ctxMenuCreateSellOrder;
    private javax.swing.JMenuItem ctxMenuModifyOder;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable table;
    // End of variables declaration//GEN-END:variables
}
