/*
 * Copyright (c) 2018, 7u83 <7u83@mail.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package opensesim.util.idgenerator;

import java.util.function.Supplier;

/**
 * Implementation of a simple ID generator to create unique IDs 
 *
 * @author 7u83 <7u83@mail.ru>
 */
public class IDGenerator<T extends Number> {

    private Long next_id ;
    private Long start_id;

    
     
    
    /**
     * Initialize the ID generator
     *
     * @param start ID value to start with
     */
    public IDGenerator(Long start) {
        start_id=start;
        reset();
    }

    /**
     * Initialize ID Generator with start ID = 0
     */
    public IDGenerator() {
        this(0L);
        
    }

    /**
     * Reset the ID generator
     */
    protected final void reset() {
        next_id = start_id;
    }

    /**
     * Get the next ID
     *
     * @return the next generated ID
     */
    public synchronized Id getNext() {
        return new Id<Long>(next_id++);
    }
}
